@@ -19,7 +19,9 @@ from pathlib import Path
 from typing import Any
 from typing import Dict
 from typing import List
+from typing import Literal
 from typing import Sequence
+from typing import Union
 
 import pandas as pd
 import zarr
@@ -33,6 +35,7 @@ from fractal_tasks_core.lib_metadata_parsing import parse_yokogawa_metadata
 from fractal_tasks_core.lib_parse_filename_metadata import parse_filename
 from fractal_tasks_core.lib_regions_of_interest import prepare_FOV_ROI_table
 from fractal_tasks_core.lib_regions_of_interest import prepare_well_ROI_table
+from fractal_tasks_core.lib_remove_FOV_overlaps import remove_FOV_overlaps
 
 
 __OME_NGFF_VERSION__ = fractal_tasks_core.__OME_NGFF_VERSION__
@@ -50,7 +53,7 @@ def create_ome_zarr_multiplex(
     allowed_channels: Dict[str, Sequence[Dict[str, Any]]],
     num_levels: int = 2,
     coarsening_xy: int = 2,
-    metadata_table: str = "mrf_mlf",
+    metadata_table: Union[Literal["mrf_mlf"], Dict[str, str]] = "mrf_mlf",
 ) -> Dict[str, Any]:
     """
     Create OME-NGFF structure and metadata to host a multiplexing dataset
@@ -67,26 +70,45 @@ def create_ome_zarr_multiplex(
     :param output_path: parent folder for the output path, e.g.
                         ``"/outputpath/*.zarr"``
     :param metadata: standard fractal argument, not used in this task
-    :param channel_parameters: TBD
+    :param allowed_channels: TBD
     :param num_levels: number of resolution-pyramid levels
-    :param coarsening_xy: linear coarsening factor between subsequent levels
-    :param metadata_table: TBD
+    :param coarsening_xy: Linear coarsening factor between subsequent levels
+    :param metadata_table: If equal to ``"mrf_mlf"``, parse Yokogawa metadata
+                           from mrf/mlf files in the ``input_paths`` folders;
+                           else, a dictionary of key-value pairs like
+                           ``(acquisition, path)`` with ``acquisition`` a
+                           string and ``path`` pointing to a csv file
+                           containing the parsed metadata table.
     """
 
     # Preliminary checks on metadata_table
-    if metadata_table != "mrf_mlf" and not isinstance(
-        metadata_table, pd.core.frame.DataFrame
-    ):
-        raise Exception(
+    if metadata_table != "mrf_mlf" and not isinstance(metadata_table, Dict):
+        raise ValueError(
             "ERROR: metadata_table must be a known string or a "
-            "pandas DataFrame}"
-        )
-    if metadata_table != "mrf_mlf":
-        raise NotImplementedError(
-            "We currently only support "
-            'metadata_table="mrf_mlf", '
-            f"and not {metadata_table}"
+            "dict of csv file containing a pandas dataframe."
+            f"The metadata_table provided was {metadata_table}"
         )
+    elif isinstance(metadata_table, Dict):
+        # Check that acquisition keys are string
+        for key, value in metadata_table.items():
+            if not isinstance(key, str):
+                raise ValueError(f"{metadata_table=} has non-string keys")
+
+        # Checks on the dict:
+        # 1. Acquisitions as keys (same as keys of allowed_channels)
+        # 2. Files end with ".csv"
+        # 3. Files exist.
+        if set(allowed_channels.keys()) != set(metadata_table.keys()):
+            raise ValueError(
+                "Mismatch in acquisition keys between "
+                f"{allowed_channels.keys()=} and {metadata_table.keys()=}"
+            )
+        if not all([x.endswith(".csv") for x in metadata_table.values()]):
+            raise ValueError(
+                f"Some files in {metadata_table=} do not end with csv."
+            )
+        if not all([os.path.isfile(x) for x in metadata_table.values()]):
+            raise ValueError(f"Some files in {metadata_table=} do not exist.")
 
     # Preliminary checks on allowed_channels
     # Note that in metadata the keys of dictionary arguments should be
@@ -214,24 +236,23 @@ def create_ome_zarr_multiplex(
         logger.info(f"Looking at {image_folder=}")
 
         # Obtain FOV-metadata dataframe
-        try:
-            if metadata_table == "mrf_mlf":
-                mrf_path = f"{image_folder}/MeasurementDetail.mrf"
-                mlf_path = f"{image_folder}/MeasurementData.mlf"
-                site_metadata, total_files = parse_yokogawa_metadata(
-                    mrf_path, mlf_path
-                )
-                has_mrf_mlf_metadata = True
+        if metadata_table == "mrf_mlf":
+            mrf_path = f"{image_folder}/MeasurementDetail.mrf"
+            mlf_path = f"{image_folder}/MeasurementData.mlf"
+            site_metadata, total_files = parse_yokogawa_metadata(
+                mrf_path, mlf_path
+            )
+            site_metadata = remove_FOV_overlaps(site_metadata)
 
-                # Extract pixel sizes and bit_depth
-                pixel_size_z = site_metadata["pixel_size_z"][0]
-                pixel_size_y = site_metadata["pixel_size_y"][0]
-                pixel_size_x = site_metadata["pixel_size_x"][0]
-                bit_depth = site_metadata["bit_depth"][0]
-        except FileNotFoundError:
-            logger.info("Missing metadata files")
-            has_mrf_mlf_metadata = False
-            pixel_size_x = pixel_size_y = pixel_size_z = 1
+        elif isinstance(metadata_table, Dict):
+            site_metadata = pd.read_csv(metadata_table[acquisition])
+            site_metadata.set_index(["well_id", "FieldIndex"], inplace=True)
+
+        # Extract pixel sizes and bit_depth
+        pixel_size_z = site_metadata["pixel_size_z"][0]
+        pixel_size_y = site_metadata["pixel_size_y"][0]
+        pixel_size_x = site_metadata["pixel_size_x"][0]
+        bit_depth = site_metadata["bit_depth"][0]
 
         if min(pixel_size_z, pixel_size_y, pixel_size_x) < 1e-9:
             raise Exception(pixel_size_z, pixel_size_y, pixel_size_x)
@@ -390,22 +411,19 @@ def create_ome_zarr_multiplex(
                 ),
             }
 
-            if has_mrf_mlf_metadata:
-                group_tables = group_image.create_group(
-                    "tables/"
-                )  # noqa: F841
+            # Create tables zarr group for ROI tables
+            group_tables = group_image.create_group("tables/")  # noqa: F841
+            well_id = row + column
 
-                # Prepare image/well tables
-                FOV_ROIs_table = prepare_FOV_ROI_table(
-                    site_metadata.loc[f"{row+column}"],
-                )
-                # Prepare and write anndata table of well ROIs
-                well_ROIs_table = prepare_well_ROI_table(
-                    site_metadata.loc[f"{row+column}"],
-                )
-                # Write tables
-                write_elem(group_tables, "FOV_ROI_table", FOV_ROIs_table)
-                write_elem(group_tables, "well_ROI_table", well_ROIs_table)
+            # Prepare AnnData tables for FOV/well ROIs
+            FOV_ROIs_table = prepare_FOV_ROI_table(site_metadata.loc[well_id])
+            well_ROIs_table = prepare_well_ROI_table(
+                site_metadata.loc[well_id]
+            )
+
+            # Write AnnData tables in the tables zarr group
+            write_elem(group_tables, "FOV_ROI_table", FOV_ROIs_table)
+            write_elem(group_tables, "well_ROI_table", well_ROIs_table)
 
     # Check that the different images (e.g. different cycles) in the each well
     # have unique labels
@@ -441,7 +459,7 @@ if __name__ == "__main__":
         allowed_channels: Dict[str, Sequence[Dict[str, Any]]]
         num_levels: int = 2
         coarsening_xy: int = 2
-        metadata_table: str = "mrf_mlf"
+        metadata_table: Union[Literal["mrf_mlf"], Dict[str, str]] = "mrf_mlf"
 
     run_fractal_task(
         task_function=create_ome_zarr_multiplex,
